Package: SimuRg
Title: Building, Fitting and Evaluating PK/PD Modeles
Version: 0.2.0
Authors@R: c(
    person("Victor", "Sokolov", , "victor.sokolov@msdecisions.tech", role =  c("cph", "aut")),
    person("Anna", "Mikhailova", , "anna.mikhailova@msdecisions.tech", role = "cre"),
    person("Yaroslav", "Ugolkov", , "yaroslav.ugolkov@msdecisions.tech", role = "aut"),
    person("Anatoly", "Pokladyuk", , "anatoly.pokladyuk@msdecisions.tech", role = "aut"),
    person("Alina", "Melnikova", , "alina.melnikova@msdecisions.tech", role = "aut"),
    person("Victoria", "Kulesh", , "victoria.kulesh@msdecisions.tech", role = "aut")
  )
Description: Provides a unified workflow for building, fitting using external engines, and evaluating ordinary differential equation (ODE)-based pharmacokinetic/pharmacodynamic (PK/PD) models. Supports generation of estimation scenarios and control files for external engines (e.g., 'Monolix'), simulation of models using 'rxode2', and creation of goodness-of-fit diagnostics. Includes tools for covariate modeling, virtual population design, and local and global sensitivity analyses.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: GGally, testthat (>= 3.0.0)
Imports: cluster, dplyr, fastDummies, forcats, ggplot2, jsonlite, MASS,
        philentropy, purrr, readr, recipes, rlang, rxode2, scales,
        stringr, synthpop, sys, tibble, tidyr, uwot, zoo, lhs,
        sensitivity, ppcor, withr
Config/testthat/edition: 3
Depends: R (>= 3.5)
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-15 11:15:10 UTC; MikhailovaAnna
Author: Victor Sokolov [cph, aut],
  Anna Mikhailova [cre],
  Yaroslav Ugolkov [aut],
  Anatoly Pokladyuk [aut],
  Alina Melnikova [aut],
  Victoria Kulesh [aut]
Maintainer: Anna Mikhailova <anna.mikhailova@msdecisions.tech>
Repository: CRAN
Date/Publication: 2026-05-19 09:40:10 UTC
Built: R 4.5.2; ; 2026-05-19 10:36:54 UTC; unix
