# Generated by roxygen2: do not edit by hand

export(read_smrg_obj)
export(sg_converter)
export(sg_covsens_sim)
export(sg_covsens_vis)
export(sg_fit)
export(sg_globalsens_sim)
export(sg_globalsens_vis)
export(sg_gof_obpr)
export(sg_gof_par_cov)
export(sg_gof_par_dist)
export(sg_gof_res)
export(sg_gof_res_dist)
export(sg_gof_tp)
export(sg_localsens_sim)
export(sg_localsens_vis)
export(sg_modbuild)
export(sg_modcomp)
export(sg_multistart)
export(sg_parsum)
export(sg_predist_sim)
export(sg_sim)
export(sg_sim_tp)
export(sg_translator)
export(sg_vpc_sim)
export(sg_vpc_vis)
export(sg_vpop_est)
import(dplyr)
import(ggplot2)
import(ppcor)
import(purrr)
import(rxode2)
import(stringr)
import(sys)
import(tibble)
import(tidyr)
importFrom(MASS,mvrnorm)
importFrom(cluster,daisy)
importFrom(fastDummies,dummy_cols)
importFrom(forcats,fct_inorder)
importFrom(grDevices,recordPlot)
importFrom(graphics,abline)
importFrom(graphics,box)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(grid,unit)
importFrom(jsonlite,fromJSON)
importFrom(lhs,randomLHS)
importFrom(philentropy,distance)
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(purrr,map_dfr)
importFrom(readr,cols)
importFrom(readr,parse_number)
importFrom(readr,read_csv)
importFrom(readr,read_tsv)
importFrom(readr,write_csv)
importFrom(recipes,bake)
importFrom(recipes,prep)
importFrom(recipes,recipe)
importFrom(recipes,step_center)
importFrom(recipes,step_dummy)
importFrom(recipes,step_scale)
importFrom(rlang,.data)
importFrom(rlang,expr)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(scales,math_format)
importFrom(scales,number_format)
importFrom(scales,pretty_breaks)
importFrom(scales,trans_breaks)
importFrom(scales,trans_format)
importFrom(sensitivity,fast99)
importFrom(sensitivity,tell)
importFrom(stats,lm)
importFrom(stats,runif)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_remove)
importFrom(synthpop,syn)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,drop_na)
importFrom(tidyr,gather)
importFrom(tidyr,separate)
importFrom(uwot,umap)
importFrom(uwot,umap_transform)
importFrom(zoo,na.locf)
