# sg-modcomp works

    Code
      sum_tab
    Output
        Run ID Project name      ETAs                        Covariates
      1      3   proj-r-cov ka, V, Cl AGE~V, CYP2C9_gentyp(5 levels)~Cl
        Identifiability Shrinkage Max RSE, % Max shrinkage, %      OFV       AIC
      1             Yes       Yes       35.5             6.04 20668.24 -10308.12

---

    Code
      sum_tab
    Output
        Run ID Project name   ETAs Covariates Identifiability Shrinkage Max RSE, %
      1      4    proj-solo ka, Cl                        Yes       Yes       7.36
        Max shrinkage, %      OFV      AIC
      1             2.86 19064.22 -9520.11

