# sg-gof-res output is correct

    Code
      ggplot2::layer_data(x)
    Output
        yintercept PANEL group colour linewidth linetype alpha
      1          0     1    -1  black       0.7   dashed    NA
      2          0     2    -1  black       0.7   dashed    NA
      3          0     3    -1  black       0.7   dashed    NA

# sg-gof-res file load

    Code
      ggplot2::layer_data(x)
    Output
        yintercept PANEL group colour linewidth linetype alpha
      1          0     1    -1  black       0.7   dashed    NA

