% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{warfarin}
\alias{warfarin}
\title{Warfarin PKPD dataset}
\format{
\subsection{\code{warfarin}}{

A data frame with 1700 rows and 17 columns:
\describe{
\item{ID}{identifier of the individual}
\item{TIME}{time of the dose or observation record (nominal)}
\item{DV}{records the measurement data}
\item{DVID}{identifier for the observation type (to distinguish different types of observations, e.g PK and PD)}
\item{DVNAME}{name for the observation type (to distinguish different types of observations, e.g PK and PD)}
\item{CMT}{compartment of the event}
\item{ADM}{identifier for the type of dose}
\item{AMT}{dose amount}
\item{EVID}{identifier to indicate if the line is a dose-line or a response-line}
\item{MDV}{identifier to ignore the observation information of that line}
\item{AGE}{age of the individual}
\item{SEX}{sex of the individual}
\item{WEIGHT}{weight of the individual}
\item{BMI}{BMI of the individual}
\item{CLCR}{creatinine clearance of the individual}
\item{CYP2C9_gentyp}{CYP2C9 genotype of the individual}
\item{VKORC1_gentyp}{VKORC1 genotype of the individual}
}
}
}
\usage{
warfarin
}
\description{
Generated dataset with warfarin PK/PD measurements and covariates for 100 patients.
}
\keyword{datasets}
