% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_pbc}
\alias{data_pbc}
\title{Primary Biliary Cirrhosis (PBC) dataset}
\format{
\subsection{\code{data_pbc}}{

A data frame with 280 rows and 15 columns:
\describe{
\item{id}{identifier of the individual}
\item{years}{time in years (survival time)}
\item{status2}{survival status (0 = censored, 1 = event)}
\item{drug}{treatment group ("D-penicil" or "placebo")}
\item{age}{age of the individual in years}
\item{sex}{sex of the individual ("male" or "female")}
\item{ascites}{presence of ascites ("Yes" or "No")}
\item{hepatomegaly}{presence of hepatomegaly ("Yes" or "No")}
\item{spiders}{presence of spider angiomas ("Yes" or "No")}
\item{edema}{edema status ("No edema", "edema no diuretics", or "edema despite diuretics")}
\item{serBilir}{serum bilirubin level (mg/dl)}
\item{serChol}{serum cholesterol level (mg/dl)}
\item{albumin}{albumin level (g/dl)}
\item{platelets}{platelet count (per cubic mm/1000)}
}
}
}
\usage{
data_pbc
}
\description{
Clinical trial dataset with Primary Biliary Cirrhosis patient data including survival outcomes and covariates.
}
\keyword{datasets}
