% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SimuRg.R
\docType{package}
\name{SimuRg-package}
\alias{SimuRg}
\alias{SimuRg-package}
\title{SimuRg: Building, fitting and evaluating PK/PD Modeles}
\description{
SimuRg is an R toolkit for PK/PD modeling and simulation workflows, with a focus on importing, organizing, and exploring nonlinear mixed-effects model results.
}
\author{
\strong{Maintainer}: Anna Mikhailova \email{anna.mikhailova@msdecisions.tech}

Authors:
\itemize{
  \item Victor Sokolov \email{victor.sokolov@msdecisions.tech} [copyright holder]
  \item Yaroslav Ugolkov \email{yaroslav.ugolkov@msdecisions.tech}
  \item Anatoly Pokladyuk \email{anatoly.pokladyuk@msdecisions.tech}
  \item Alina Melnikova \email{alina.melnikova@msdecisions.tech}
  \item Victoria Kulesh \email{victoria.kulesh@msdecisions.tech}
}

}
\keyword{internal}
